/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


const Cc = Components.classes;
const Ci = Components.interfaces;

const TYPE_DIRECT  = 1;
const TYPE_REGEXP  = 2;
const TYPE_CUSTOM  = 3;
const TYPE_WRAPPED = 4;

const TYPE_REGEXP_THUMBNAIL = 1;
const TYPE_REGEXP_LINK      = 2;

const TYPE_INDEX = [0, 1, 3, 2];

const TYPE_ELEMENT_IDS =
    [[], 
     ["box_regexp_source", "row_regexp_pattern", "row_regexp_template"],
     ["row_custom_code"],
     ["row_wrapped_attr_name", "row_wrapped_attr_value"]];
     

const TEMPLATE_URIS =
{
    imageId:     "chrome://thumbsdown/content/templates/img_id.js",
    imageRegExp: "chrome://thumbsdown/content/templates/img_attr_regexp.js",
    frameRedirect: "chrome://thumbsdown/content/templates/frame_redirect.js",
}

var currentType = 0;


function onLoad()
{
    stringBundle = $("stringbundle");
    $("labelColumn").style.width = _("css.labelColumn.width");
    var script = window.arguments[0].arg;
    if (script)
    {
        $("tbox_name").value = script.name;
        $("tbox_matcher").value = script.matcher;
        $("mlist_type").selectedIndex = TYPE_INDEX[script.type - 1];
        
        switch (script.type)
        {
            case TYPE_REGEXP:
                $("radio_regexp_source").value = script.source;
                $("tbox_regexp_pattern").value = script.pattern;
                $("tbox_regexp_template").value = script.template;
                break;
            case TYPE_CUSTOM:
                $("tbox_custom_code").value = script.script;
                break;
            case TYPE_WRAPPED:
                $("tbox_wrapped_attr_name").value = script.attrName;
                $("tbox_wrapped_attr_value").value = script.attrValue;
                break;
        }
        
        currentType = script.type;
        _showTypeElements(currentType, true)
    }
}

function onTypeChange()
{
    // avoid unneccessary sizeToContent() calls in ff3 - https://bugzilla.mozilla.org/show_bug.cgi?id=439323#c4
    var newType = $("mlist_type").value;
    if (newType == currentType)
        return;
    _showTypeElements(currentType, false);
    currentType = newType;
    _showTypeElements(currentType, true);
}

function useScriptTemplate(templateId)
{
    var scriptTemplate = loadContent(TEMPLATE_URIS[templateId]);
    var tbox = $("tbox_custom_code");
    tbox.value = scriptTemplate;
}

function onDialogAccept()
{
    try
    {
        var result = {}
        result.name = trim($("tbox_name").value);
        result.matcher = trim($("tbox_matcher").value);
        result.type = Number($("mlist_type").value);
        
        if (result.name.length == 0) throw _("error.empty", _("name"));
        if (result.matcher.length == 0) throw _("error.empty", _("matcher"));
        if (! _isValidRegExp("^" + result.matcher + "$")) throw _("error.noRegExp", _("matcher"));
        
        switch (result.type)
        {
            case TYPE_REGEXP:
                result.subType = Number($("radio_regexp_source").value);
                result.action1 = trim($("tbox_regexp_pattern").value);
                result.action2 = trim($("tbox_regexp_template").value);
                if (result.action1.length == 0) throw _("error.empty", _("regexp.pattern"));
                if (result.action2.length == 0) throw _("error.empty", _("regexp.template"));
                if (! _isValidRegExp("^" + result.action1 + "$")) throw _("error.noRegExp", _("regexp.pattern"));
                break;
            
            case TYPE_CUSTOM:
                result.action1 = trim($("tbox_custom_code").value);
                if (result.action1.length == 0) throw _("error.empty", _("custom.code"));
                break;
            
            case TYPE_WRAPPED:
                result.action1 = trim($("tbox_wrapped_attr_name").value);
                result.action2 = trim($("tbox_wrapped_attr_value").value);
                if (result.action1.length == 0) throw _("error.empty", _("wrapped.attr.name"));
                if (result.action2.length == 0) throw _("error.empty", _("wrapped.attr.value"));
                if (! _isValidRegExp("^" + result.action2 + "$")) throw _("error.noRegExp", _("wrapped.attr.value"));
                break;
        }
        
        window.arguments[0].result = result;
        return true;
    }
    catch (e if (typeof(e) == "string"))
    {
        var promptSvc = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
        promptSvc.alert(window, _("error.title"), e);
        return false;
    }
}


function _showTypeElements(type, show)
{
    var ids = TYPE_ELEMENT_IDS[type - 1];
    for each (var id in ids)
        $(id).hidden = (! show);
    if (show)
        window.sizeToContent();
}

function _isValidRegExp(pattern)
{
    try
    {
        new RegExp(pattern);
        return true;
    }
    catch (e)
    {
        return false;
    }
}
